window.Gleam.initSearch([{"doc":"decode","title":"decode","content":"# decode\n\nErgonomic dynamic decoders for Gleam!\n\n[![Package Version](https://img.shields.io/hexpm/v/decode)](https://hex.pm/packages/decode)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/decode/)\n\n```sh\ngleam add decode\n```\n```gleam\nimport decode/zero as decode\nimport gleam/dynamic.{type Dynamic}\n\n\npub type User {\n  User(name: String, email: String, is_admin: Bool)\n}\n\n/// Decode data of this shape into a `User` record.\n///\n/// {\n///   &quot;name&quot; -&gt; &quot;Lucy&quot;,\n///   &quot;email&quot; -&gt; &quot;lucy@example.com&quot;,\n///   &quot;is-admin&quot; -&gt; true\n/// }\n///\npub fn run(data: Dynamic) {\n  let decoder = {\n    use name &lt;- decode.field(&quot;name&quot;, decode.string)\n    use email &lt;- decode.field(&quot;score&quot;, decode.string)\n    use is_admin &lt;- decode.field(&quot;enrolled&quot;, decode.bool)\n    decode.success(User(name:, email:, is_admin:))\n  }\n\n  decode.run(data, decoder)\n}\n```\nOr, alternatively:\n```gleam\nimport decode\nimport gleam/dynamic.{type Dynamic}\n\npub type User {\n  User(name: String, email: String, is_admin: Bool)\n}\n\n/// Decode data of this shape into a `User` record.\n///\n/// {\n///   &quot;name&quot; -&gt; &quot;Lucy&quot;,\n///   &quot;email&quot; -&gt; &quot;lucy@example.com&quot;,\n///   &quot;is-admin&quot; -&gt; true\n/// }\n///\npub fn run(data: Dynamic) {\n  let decoder =\n    decode.into({\n      use name &lt;- decode.parameter\n      use email &lt;- decode.parameter\n      use is_admin &lt;- decode.parameter\n      User(name, email, is_admin)\n    })\n    |&gt; decode.field(&quot;name&quot;, decode.string)\n    |&gt; decode.field(&quot;email&quot;, decode.string)\n    |&gt; decode.field(&quot;is-admin&quot;, decode.bool)\n\n  decoder\n  |&gt; decode.from(data)\n}\n```\nFor more documentation and examples view &lt;https://hexdocs.pm/decode&gt;.\n","url":"index.html"},{"doc":"decode","title":"DecodeResult","content":"pub type DecodeResult(t) =\n  Result(t, List(dynamic.DecodeError))\n The result that a decoder returns when run.\n\nSynonyms:\ndecode.DecodeResult\ndecode DecodeResult","url":"decode.html#DecodeResult"},{"doc":"decode","title":"Decoder","content":"pub opaque type Decoder(t)\n A decoder is a value that can be used to turn dynamically typed `Dynamic`\n data into typed data using the `from` function.\n\n Several smaller decoders can be combined to make larger decoders using\n functions such as `list` and `field`.\n\n\nSynonyms:\ndecode.Decoder\ndecode Decoder","url":"decode.html#Decoder"},{"doc":"decode","title":"bit_array","content":"pub const bit_array: Decoder(BitArray)\n A decoder that decodes `BitArray` values. This decoder never returns an error.\n\n # Examples\n\n \n decode.bit_array\n |&gt; decode.from(dynamic.from(&lt;&lt;5, 7&gt;&gt;))\n // -&gt; Ok(&lt;&lt;5, 7&gt;&gt;)\n \n\nSynonyms:\ndecode.bit_array\ndecode bit_array","url":"decode.html#bit_array"},{"doc":"decode","title":"bool","content":"pub const bool: Decoder(Bool)\n A decoder that decodes `Bool` values.\n\n # Examples\n\n \n decode.bool\n |&gt; decode.from(dynamic.from(True))\n // -&gt; Ok(True)\n \n\nSynonyms:\ndecode.bool\ndecode bool","url":"decode.html#bool"},{"doc":"decode","title":"dynamic","content":"pub const dynamic: Decoder(Dynamic)\n A decoder that decodes `Dynamic` values. This decoder never returns an error.\n\n # Examples\n\n \n decode.dynamic\n |&gt; decode.from(dynamic.from(3.14))\n // -&gt; Ok(dynamic.from(3.14))\n \n\nSynonyms:\ndecode.dynamic\ndecode dynamic","url":"decode.html#dynamic"},{"doc":"decode","title":"float","content":"pub const float: Decoder(Float)\n A decoder that decodes `Float` values.\n\n # Examples\n\n \n decode.float\n |&gt; decode.from(dynamic.from(3.14))\n // -&gt; Ok(3.14)\n \n\nSynonyms:\ndecode.float\ndecode float","url":"decode.html#float"},{"doc":"decode","title":"int","content":"pub const int: Decoder(Int)\n A decoder that decodes `Int` values.\n\n # Examples\n\n \n decode.int\n |&gt; decode.from(dynamic.from(147))\n // -&gt; Ok(147)\n \n\nSynonyms:\ndecode.int\ndecode int","url":"decode.html#int"},{"doc":"decode","title":"string","content":"pub const string: Decoder(String)\n A decoder that decodes `String` values.\n\n # Examples\n\n \n decode.string\n |&gt; decode.from(dynamic.from(&quot;Hello!&quot;))\n // -&gt; Ok(&quot;Hello!&quot;)\n \n\nSynonyms:\ndecode.string\ndecode string","url":"decode.html#string"},{"doc":"decode","title":"at","content":"pub fn at(path: List(a), inner: Decoder(b)) -&gt; Decoder(b)\n A decoder that decodes a value that is nested within other values. For\n example, decoding a value that is within some deeply nested JSON objects.\n\n This function will index into dictionaries with any key type, and if the key is\n an int then it&#39;ll also index into Erlang tuples and JavaScript arrays.\n\n # Examples\n\n \n let data = dynamic.from(dict.from_list([\n   #(&quot;one&quot;, dict.from_list([\n     #(&quot;two&quot;, 1000),\n   ])),\n ]))\n\n decode.at([&quot;one&quot;, &quot;two&quot;], decode.int)\n |&gt; decode.from(data)\n // -&gt; Ok(1000)\n \n\n \n decode.optional(decode.int)\n |&gt; decode.from(dynamic.from(Nil))\n // -&gt; Ok(option.None)\n \n\nSynonyms:\ndecode.at\ndecode at","url":"decode.html#at"},{"doc":"decode","title":"collapse_errors","content":"pub fn collapse_errors(\n  decoder: Decoder(a),\n  name: String,\n) -&gt; Decoder(a)\n Replace all errors produced by a decoder with one single error for a named\n expected type.\n\n This function may be useful if you wish to simplify errors before\n presenting them to a user, particularly when using the `one_of` function.\n\n # Examples\n\n \n decode.string\n |&gt; decode.collapse_errors(&quot;MyThing&quot;)\n |&gt; decode.from(dynamic.from(1000))\n // -&gt; Error([DecodeError(&quot;MyThing&quot;, &quot;Int&quot;, [])])\n \n\nSynonyms:\ndecode.collapse_errors\ndecode collapse_errors","url":"decode.html#collapse_errors"},{"doc":"decode","title":"dict","content":"pub fn dict(\n  key: Decoder(a),\n  value: Decoder(b),\n) -&gt; Decoder(Dict(a, b))\n A decoder that decodes dicts where all keys and vales are decoded with\n given decoders.\n\n # Examples\n\n \n let values = dict.from_list([\n   #(&quot;one&quot;, 1),\n   #(&quot;two&quot;, 2),\n ])\n decode.dict(decode.string, decode.int)\n |&gt; decode.from(dynamic.from(values))\n // -&gt; Ok(values)\n \n\nSynonyms:\ndecode.dict\ndecode dict","url":"decode.html#dict"},{"doc":"decode","title":"fail","content":"pub fn fail(expected: String) -&gt; Decoder(a)\n Define a decoder that always fails. The parameter for this function is the\n name of the type that has failed to decode.\n\nSynonyms:\ndecode.fail\ndecode fail","url":"decode.html#fail"},{"doc":"decode","title":"field","content":"pub fn field(\n  decoder: Decoder(fn(a) -&gt; b),\n  field_name: c,\n  field_decoder: Decoder(a),\n) -&gt; Decoder(b)\n Run a decoder on a `Dynamic` value, decoding the value if it is of the\n desired type, or returning errors.\n\n The second parameter is a field name which will be used to index into the\n `Dynamic` data.\n\n This function will index into dictionaries with any key type, and if the key is\n an int then it&#39;ll also index into Erlang tuples and JavaScript arrays.\n\n # Examples\n\n \n let data = dynamic.from(dict.from_list([\n   #(&quot;email&quot;, &quot;lucy@example.com&quot;),\n   #(&quot;name&quot;, &quot;Lucy&quot;),\n ]))\n\n decode.into({\n   use name &lt;- decode.parameter\n   use email &lt;- decode.parameter\n   SignUp(name: name, email: email)\n })\n |&gt; decode.field(&quot;name&quot;, decode.string)\n |&gt; decode.field(&quot;email&quot;, decode.string)\n |&gt; decode.from(data)\n // -&gt; Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \n\n If you wish to decode a value that is more deeply nested within the dynamic\n data, see [`subfield`](#subfield) and [`at`](#at).\n\nSynonyms:\ndecode.field\ndecode field","url":"decode.html#field"},{"doc":"decode","title":"from","content":"pub fn from(\n  decoder: Decoder(a),\n  data: Dynamic,\n) -&gt; Result(a, List(DecodeError))\n Run a decoder on a `Dynamic` value, decoding the value if it is of the\n desired type, or returning errors.\n\n # Examples\n\n \n decode.into({\n   use name &lt;- decode.parameter\n   use email &lt;- decode.parameter\n   SignUp(name: name, email: email)\n })\n |&gt; decode.field(&quot;email&quot;, decode.string)\n |&gt; decode.field(&quot;password&quot;, decode.string)\n |&gt; decode.from(data)\n \n\nSynonyms:\ndecode.from\ndecode from","url":"decode.html#from"},{"doc":"decode","title":"into","content":"pub fn into(constructor: a) -&gt; Decoder(a)\n Create a new decoder for a given constructor function. If this function is\n a function that takes parameters one-at-a-time, such as anonymous functions\n made with `use name &lt;- decode.parameter`, then the decoder can be used with\n the `decode.field` function to decode a value that contains multiple other\n values.\n\n # Examples\n\n \n let data = dynamic.from(dict.from_list([\n   #(&quot;email&quot;, &quot;lucy@example.com&quot;),\n   #(&quot;name&quot;, &quot;Lucy&quot;),\n ]))\n\n decode.into({\n   use name &lt;- decode.parameter\n   use email &lt;- decode.parameter\n   SignUp(name: name, email: email)\n })\n |&gt; decode.field(&quot;name&quot;, decode.string)\n |&gt; decode.field(&quot;email&quot;, decode.string)\n |&gt; decode.from(data)\n // -&gt; Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \n\nSynonyms:\ndecode.into\ndecode into","url":"decode.html#into"},{"doc":"decode","title":"list","content":"pub fn list(of item: Decoder(a)) -&gt; Decoder(List(a))\n A decoder that decodes lists where all elements are decoded with a given\n decoder.\n\n # Examples\n\n \n decode.list(of: decode.int)\n |&gt; decode.from(dynamic.from([1, 2, 3]))\n // -&gt; Ok([1, 2, 3])\n \n\nSynonyms:\ndecode.list\ndecode list","url":"decode.html#list"},{"doc":"decode","title":"map","content":"pub fn map(\n  decoder: Decoder(a),\n  transformer: fn(a) -&gt; b,\n) -&gt; Decoder(b)\n Apply a transformation function to any value decoded by the decoder.\n\n # Examples\n\n \n decode.int\n |&gt; decode.map(int.to_string)\n |&gt; decode.from(dynamic.from(1000))\n // -&gt; Ok(&quot;1000&quot;)\n \n\nSynonyms:\ndecode.map\ndecode map","url":"decode.html#map"},{"doc":"decode","title":"map_errors","content":"pub fn map_errors(\n  decoder: Decoder(a),\n  transformer: fn(List(DecodeError)) -&gt; List(DecodeError),\n) -&gt; Decoder(a)\n Apply a transformation function to any errors returned by the decoder.\n\nSynonyms:\ndecode.map_errors\ndecode map_errors","url":"decode.html#map_errors"},{"doc":"decode","title":"one_of","content":"pub fn one_of(decoders: List(Decoder(a))) -&gt; Decoder(a)\n Create a new decoder from several other decoders. Each of the inner\n decoders is run in turn, and the value from the first to succeed is used.\n\n If no decoder succeeds then the errors from the final decoder is used.\n If you wish for different errors then you may wish to use the\n `collapse_errors` or `map_errors` functions.\n\n # Examples\n\n \n decode.one_of([\n   decode.string,\n   decode.int |&gt; decode.map(int.to_string),\n ])\n |&gt; decode.from(dynamic.from(1000))\n // -&gt; Ok(&quot;1000&quot;)\n \n\nSynonyms:\ndecode.one_of\ndecode one_of","url":"decode.html#one_of"},{"doc":"decode","title":"optional","content":"pub fn optional(item: Decoder(a)) -&gt; Decoder(Option(a))\n A decoder that decodes nullable values of a type decoded by with a given\n decoder.\n\n This function can handle common representations of null on all runtimes, such as\n `nil`, `null`, and `undefined` on Erlang, and `undefined` and `null` on\n JavaScript.\n\n # Examples\n\n \n decode.optional(decode.int)\n |&gt; decode.from(dynamic.from(100))\n // -&gt; Ok(option.Some(100))\n \n\n \n decode.optional(decode.int)\n |&gt; decode.from(dynamic.from(Nil))\n // -&gt; Ok(option.None)\n \n\nSynonyms:\ndecode.optional\ndecode optional","url":"decode.html#optional"},{"doc":"decode","title":"parameter","content":"pub fn parameter(body: fn(a) -&gt; b) -&gt; fn(a) -&gt; b\n This function is used to create constructor functions that take arguments\n one at a time, making them suitable for passing to the `into` function.\n\n # Examples\n\n \n let data = dynamic.from(dict.from_list([\n   #(&quot;email&quot;, &quot;lucy@example.com&quot;),\n   #(&quot;name&quot;, &quot;Lucy&quot;),\n ]))\n\n decode.into({\n   use name &lt;- decode.parameter\n   use email &lt;- decode.parameter\n   SignUp(name: name, email: email)\n })\n |&gt; decode.field(&quot;name&quot;, decode.string)\n |&gt; decode.field(&quot;email&quot;, decode.string)\n |&gt; decode.from(data)\n // -&gt; Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \n\nSynonyms:\ndecode.parameter\ndecode parameter","url":"decode.html#parameter"},{"doc":"decode","title":"subfield","content":"pub fn subfield(\n  decoder: Decoder(fn(a) -&gt; b),\n  field_path: List(c),\n  field_decoder: Decoder(a),\n) -&gt; Decoder(b)\n The same as [`field`](#field), except taking a path to the value rather\n than a field name.\n\n # Examples\n\n \n let data = dynamic.from(dict.from_list([\n   #(&quot;data&quot;, dict.from_list([\n     #(&quot;email&quot;, &quot;lucy@example.com&quot;),\n     #(&quot;name&quot;, &quot;Lucy&quot;),\n   ]))\n ]))\n\n decode.into({\n   use name &lt;- decode.parameter\n   use email &lt;- decode.parameter\n   SignUp(name: name, email: email)\n })\n |&gt; decode.subfield([&quot;data&quot;, &quot;name&quot;], decode.string)\n |&gt; decode.subfield([&quot;data&quot;, &quot;email&quot;], decode.string)\n |&gt; decode.from(data)\n // -&gt; Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \n\nSynonyms:\ndecode.subfield\ndecode subfield","url":"decode.html#subfield"},{"doc":"decode","title":"then","content":"pub fn then(\n  decoder: Decoder(a),\n  next: fn(a) -&gt; Decoder(b),\n) -&gt; Decoder(b)\n Create a new decoder based upon the value of a previous decoder.\n\n This may be useful for when you need to know some of the structure of the\n dynamic value in order to know how to decode the rest of it.\n\nSynonyms:\ndecode.then\ndecode then","url":"decode.html#then"},{"doc":"decode","title":"decode","content":" The `Dynamic` type is used to represent dynamically typed data. That is, data\n that we don&#39;t know the precise type of yet, so we need to introspect the data to\n see if it is of the desired type before we can use it. Typically data like this\n would come from user input or from untyped languages such as Erlang or JavaScript.\n\n This module provides the `Decoder` type and associated functions, which provides\n a type-safe and composable way to convert dynamic data into some desired type,\n or into errors if the data doesn&#39;t have the desired structure.\n\n The `Decoder` type is generic and has 1 type parameter, which is the type that\n it attempts to decode. A `Decoder(String)` can be used to decode strings, and a\n `Decoder(Option(Int))` can be used to decode `Option(Int)`s\n\n Decoders work using _runtime reflection_ and the data structures of the target\n platform. Differences between Erlang and JavaScript data structures may impact\n your decoders, so it is important to test your decoders on all supported\n platforms.\n\n # Examples\n\n Dynamic data may come from various sources and so many different syntaxes could\n be used to describe or construct them. In these examples the JSON syntax is\n largely used, and you can apply the same techniques to data from any source.\n\n ## Simple types\n\n This module defines decoders for simple data types such as [`string`](#string),\n [`int`](#int), [`float`](#float), [`bit_array`](#bit_array), and [`bool`](#bool).\n\n ```gleam\n // Data:\n // &quot;Hello, Joe!&quot;\n\n let result =\n   decode.string\n   |&gt; decode.from(data)\n\n let assert Ok(&quot;Hello, Joe!&quot;) = result\n ```\n\n ## Lists\n\n The [`list`](#list) decoder decodes `List`s. To use it you must construct it by\n passing in another decoder into the `list` function, which is the decoder that\n is to be used for the elements of the list, type checking both the list and its\n elements.\n\n ```gleam\n // Data:\n // [1, 2, 3, 4]\n\n let result =\n   decode.list(decode.int)\n   |&gt; decode.from(data)\n\n let assert Ok([1, 2, 3, 4]) = result\n ```\n\n On Erlang this decoder can decode from lists, and on JavaScript it can decode\n from lists as well as JavaScript arrays.\n\n ## Options\n\n The [`optional`](#optional) decoder is used to decode values that may or may not\n be present. In other environment these might be called &quot;nullable&quot; values.\n\n Like the `list` decoder, the `optional` decoder takes another decoder,\n which is used to decode the value if it is present.\n\n ```gleam\n // Data:\n // 12.45\n\n let result =\n   decode.optional(decode.int)\n   |&gt; decode.from(data)\n\n let assert Ok(option.Some(12.45)) = result\n ```\n ```gleam\n // Data:\n // null\n\n let result =\n   decode.optional(decode.int)\n   |&gt; decode.from(data)\n\n let assert Ok(option.None) = result\n ```\n\n This decoder knows how to handle multiple different runtime representations of\n absent values, including `Nil`, `None`, `null`, and `undefined`.\n\n ## Dicts\n\n The [`dict`](#dict) decoder decodes `Dicts` and contains two other decoders, one\n for the keys, one for the values.\n\n ```gleam\n // Data:\n // { &quot;Lucy&quot;: 10, &quot;Nubi&quot;: 20 }\n\n let result =\n   decode.dict(decode.string, decode.int)\n   |&gt; decode.from(data)\n\n let assert Ok(dict.from_list([#(&quot;Lucy&quot;, 10), #(&quot;Nubi&quot;: 20)])) = result\n ```\n\n ## Indexing objects\n\n The [`at`](#at) decoder can be used to decode a value that is nested within\n key-value containers such as Gleam dicts, Erlang maps, or JavaScript objects.\n\n ```gleam\n // Data:\n // { &quot;one&quot;: { &quot;two&quot;: 123 } }\n\n let result =\n   decode.at([&quot;one&quot;, &quot;two&quot;], decode.int)\n   |&gt; decode.from(data)\n\n let assert Ok(123) = result\n ```\n\n ## Indexing arrays\n\n If you use ints as keys then the [`at`](#at) decoder can be used to index into\n array-like containers such as Gleam or Erlang tuples, or JavaScript arrays.\n\n ```gleam\n // Data:\n // [&quot;one&quot;, &quot;two&quot;, &quot;three&quot;]\n\n let result =\n   decode.at([1], decode.string)\n   |&gt; decode.from(data)\n\n let assert Ok(&quot;two&quot;) = result\n ```\n\n ## Records\n\n Decoding records from dynamic data is more complex and requires combining a\n decoder for each field and a special constructor that builds your records with\n the decoded field values.\n\n ```gleam\n // Data:\n // {\n //   &quot;score&quot;: 180,\n //   &quot;name&quot;: &quot;Mel Smith&quot;,\n //   &quot;is-admin&quot;: false,\n //   &quot;enrolled&quot;: true,\n //   &quot;colour&quot;: &quot;Red&quot;,\n // }\n\n let result =\n   decode.into({\n     use name &lt;- decode.parameter\n     use score &lt;- decode.parameter\n     use colour &lt;- decode.parameter\n     use enrolled &lt;- decode.parameter\n     Player(name: name, score: score, colour: colour, enrolled: enrolled)\n   })\n   |&gt; decode.field(&quot;name&quot;, decode.string)\n   |&gt; decode.field(&quot;score&quot;, decode.int)\n   |&gt; decode.field(&quot;colour&quot;, decode.string)\n   |&gt; decode.field(&quot;enrolled&quot;, decode.bool)\n   |&gt; decode.from(data)\n\n let assert Ok(Player(&quot;Mel Smith&quot;, 180, &quot;Red&quot;, True)) = result\n ```\n\n The ordering of the parameters defined with the `parameter` function must match\n the ordering of the decoders used with the `field` function.\n\n ## Enum variants\n\n Imagine you have a custom type where all the variants do not contain any values.\n\n ```gleam\n pub type PocketMonsterType {\n   Fire\n   Water\n   Grass\n   Electric\n }\n ```\n\n You might choose to encode these variants as strings, `&quot;fire&quot;` for `Fire`,\n `&quot;water&quot;` for `Water`, and so on. To decode them you&#39;ll need to decode the dynamic\n data as a string, but then you&#39;ll need to decode it further still as not all\n strings are valid values for the enum. This can be done with the `then`\n function, which enables running a second decoder after the first one\n succeeds.\n\n ```gleam\n let decoder =\n   decode.string\n   |&gt; decode.then(fn(decoded_string) {\n     case decoded_string {\n       // Return succeeding decoders for valid strings\n       &quot;fire&quot; -&gt; decode.into(Fire)\n       &quot;water&quot; -&gt; decode.into(Water)\n       &quot;grass&quot; -&gt; decode.into(Grass)\n       &quot;electric&quot; -&gt; decode.into(Electric)\n       // Return a failing decoder for any other strings\n       _ -&gt; decode.fail(&quot;PocketMonsterType&quot;)\n     }\n   })\n\n decoder\n |&gt; decode.from(dynamic.from(&quot;water&quot;))\n // -&gt; Ok(Water)\n\n decoder\n |&gt; decode.from(dynamic.from(&quot;wobble&quot;))\n // -&gt; Error([DecodeError(&quot;PocketMonsterType&quot;, &quot;String&quot;, [])])\n ```\n\n ## Record variants\n\n Decoding type variants that contain other values is done by combining the\n techniques from the &quot;enum variants&quot; and &quot;records&quot; examples. Imagine you have\n this custom type that you want to decode:\n\n ```gleam\n pub type PocketMonsterPerson {\n   Trainer(name: String, badge_count: Int)\n   GymLeader(name: String, speciality: PocketMonsterType)\n }\n ```\n And you would like to be able to decode these from JSON documents like these.\n ```json\n {\n   &quot;type&quot;: &quot;trainer&quot;,\n   &quot;name&quot;: &quot;Ash&quot;,\n   &quot;badge-count&quot;: 1,\n }\n ```\n ```json\n {\n   &quot;type&quot;: &quot;gym-leader&quot;,\n   &quot;name&quot;: &quot;Misty&quot;,\n   &quot;speciality&quot;: &quot;water&quot;,\n }\n ```\n\n Notice how both documents have a `&quot;type&quot;` field, which is used to indicate which\n variant the data is for.\n\n First, define decoders for each of the variants:\n\n ```gleam\n let trainer_decoder =\n   decode.into({\n     use name &lt;- decode.parameter\n     use badge_count &lt;- decode.parameter\n     Trainer(name, badge_count)\n   })\n   |&gt; decode.field(&quot;name&quot;, decode.string)\n   |&gt; decode.field(&quot;badge-count&quot;, decode.int)\n\n let gym_leader_decoder =\n   decode.into({\n     use name &lt;- decode.parameter\n     use speciality &lt;- decode.parameter\n     GymLeader(name, speciality)\n   })\n   |&gt; decode.field(&quot;name&quot;, decode.string)\n   |&gt; decode.field(&quot;speciality&quot;, pocket_monster_type_decoder)\n ```\n\n A third decoder can be used to extract and decode the `&quot;type&quot;` field, and the\n `then` function then returns whichever decoder is suitable for the document.\n\n ```gleam\n let decoder =\n   decode.at([&quot;type&quot;], decode.string)\n   |&gt; decode.then(fn(tag) {\n     case tag {\n       &quot;trainer&quot; -&gt; trainer_decoder\n       &quot;gym-leader&quot; -&gt; gym_leader\n       _ -&gt; decode.fail(&quot;PocketMonsterPerson&quot;)\n     }\n   })\n\n decoder\n |&gt; decode.from(data)\n ```","url":"decode.html"},{"doc":"decode/zero","title":"Decoder","content":"pub opaque type Decoder(t)\n A decoder is a value that can be used to turn dynamically typed `Dynamic`\n data into typed data using the `run` function.\n\n Several smaller decoders can be combined to make larger decoders using\n functions such as `list` and `field`.\n\n\nSynonyms:\ndecode/zero.Decoder\ndecode/zero Decoder","url":"decode/zero.html#Decoder"},{"doc":"decode/zero","title":"bit_array","content":"pub const bit_array: Decoder(BitArray)\n A decoder that decodes `BitArray` values. This decoder never returns an error.\n\n # Examples\n\n \n let result = zero.run(dynamic.from(&lt;&lt;5, 7&gt;&gt;), zero.bit_array)\n assert result == Ok(&lt;&lt;5, 7&gt;&gt;)\n \n\nSynonyms:\ndecode/zero.bit_array\ndecode/zero bit_array","url":"decode/zero.html#bit_array"},{"doc":"decode/zero","title":"bool","content":"pub const bool: Decoder(Bool)\n A decoder that decodes `Bool` values.\n\n # Examples\n\n \n let result = zero.run(dynamic.from(True), zero.bool)\n assert result == Ok(True)\n \n\nSynonyms:\ndecode/zero.bool\ndecode/zero bool","url":"decode/zero.html#bool"},{"doc":"decode/zero","title":"dynamic","content":"pub const dynamic: Decoder(Dynamic)\n A decoder that decodes `Dynamic` values. This decoder never returns an error.\n\n # Examples\n\n \n let result = zero.run(dynamic.from(3.14), zero.dynamic)\n assert result == Ok(dynamic.from(3.14))\n \n\nSynonyms:\ndecode/zero.dynamic\ndecode/zero dynamic","url":"decode/zero.html#dynamic"},{"doc":"decode/zero","title":"float","content":"pub const float: Decoder(Float)\n A decoder that decodes `Float` values.\n\n # Examples\n\n \n let result = zero.run(dynamic.from(3.14), zero.float)\n assert result == Ok(3.14)\n \n\nSynonyms:\ndecode/zero.float\ndecode/zero float","url":"decode/zero.html#float"},{"doc":"decode/zero","title":"int","content":"pub const int: Decoder(Int)\n A decoder that decodes `Int` values.\n\n # Examples\n\n \n let result = zero.run(dynamic.from(147), zero.int)\n assert result == Ok(147)\n \n\nSynonyms:\ndecode/zero.int\ndecode/zero int","url":"decode/zero.html#int"},{"doc":"decode/zero","title":"string","content":"pub const string: Decoder(String)\n A decoder that decodes `String` values.\n\n # Examples\n\n \n let result = zero.run(dynamic.from(&quot;Hello!&quot;), zero.string)\n assert result == Ok(&quot;Hello!&quot;)\n \n\nSynonyms:\ndecode/zero.string\ndecode/zero string","url":"decode/zero.html#string"},{"doc":"decode/zero","title":"at","content":"pub fn at(path: List(a), inner: Decoder(b)) -&gt; Decoder(b)\n A decoder that decodes a value that is nested within other values. For\n example, decoding a value that is within some deeply nested JSON objects.\n\n This function will index into dictionaries with any key type, and if the key is\n an int then it&#39;ll also index into Erlang tuples and JavaScript arrays, and\n the first two elements of Gleam lists.\n\n # Examples\n\n \n let decoder = zero.at([&quot;one&quot;, &quot;two&quot;], zero.int)\n\n let data = dynamic.from(dict.from_list([\n   #(&quot;one&quot;, dict.from_list([\n     #(&quot;two&quot;, 1000),\n   ])),\n ]))\n\n\n zero.run(data, decoder)\n // -&gt; Ok(1000)\n \n\n \n dynamic.from(Nil)\n |&gt; zero.run(zero.optional(zero.int))\n // -&gt; Ok(option.None)\n \n\nSynonyms:\ndecode/zero.at\ndecode/zero at","url":"decode/zero.html#at"},{"doc":"decode/zero","title":"collapse_errors","content":"pub fn collapse_errors(\n  decoder: Decoder(a),\n  name: String,\n) -&gt; Decoder(a)\n Replace all errors produced by a decoder with one single error for a named\n expected type.\n\n This function may be useful if you wish to simplify errors before\n presenting them to a user, particularly when using the `one_of` function.\n\n # Examples\n\n \n let decoder = zero.string |&gt; zero.collapse_errors(&quot;MyThing&quot;)\n let result = zero.run(dynamic.from(1000), decoder)\n assert result == Error([DecodeError(&quot;MyThing&quot;, &quot;Int&quot;, [])])\n \n\nSynonyms:\ndecode/zero.collapse_errors\ndecode/zero collapse_errors","url":"decode/zero.html#collapse_errors"},{"doc":"decode/zero","title":"decode_error","content":"pub fn decode_error(\n  expected expected: String,\n  found found: Dynamic,\n) -&gt; List(DecodeError)\n Construct a decode error for some unexpected dynamic data.\n\nSynonyms:\ndecode/zero.decode_error\ndecode/zero decode_error","url":"decode/zero.html#decode_error"},{"doc":"decode/zero","title":"dict","content":"pub fn dict(\n  key: Decoder(a),\n  value: Decoder(b),\n) -&gt; Decoder(Dict(a, b))\n A decoder that decodes dicts where all keys and vales are decoded with\n given decoders.\n\n # Examples\n\n \n let values = dict.from_list([\n   #(&quot;one&quot;, 1),\n   #(&quot;two&quot;, 2),\n ])\n\n let result =\n   zero.run(dynamic.from(values), zero.dict(zero.string, zero.int))\n assert result == Ok(values)\n \n\nSynonyms:\ndecode/zero.dict\ndecode/zero dict","url":"decode/zero.html#dict"},{"doc":"decode/zero","title":"failure","content":"pub fn failure(zero: a, expected: String) -&gt; Decoder(a)\n Define a decoder that always fails. The parameter for this function is the\n name of the type that has failed to zero.\n\nSynonyms:\ndecode/zero.failure\ndecode/zero failure","url":"decode/zero.html#failure"},{"doc":"decode/zero","title":"field","content":"pub fn field(\n  field_name: a,\n  field_decoder: Decoder(b),\n  next: fn(b) -&gt; Decoder(c),\n) -&gt; Decoder(c)\n Run a decoder on a field of a `Dynamic` value, decoding the value if it is\n of the desired type, or returning errors. An error is returned if there is\n no field for the specified key.\n\n This function will index into dictionaries with any key type, and if the key is\n an int then it&#39;ll also index into Erlang tuples and JavaScript arrays, and\n the first two elements of Gleam lists.\n\n # Examples\n\n \n let data = dynamic.from(dict.from_list([\n   #(&quot;email&quot;, &quot;lucy@example.com&quot;),\n   #(&quot;name&quot;, &quot;Lucy&quot;),\n ]))\n\n let decoder = {\n   use name &lt;- zero.field(&quot;name&quot;, string)\n   use email &lt;- zero.field(&quot;email&quot;, string)\n   SignUp(name: name, email: email)\n }\n\n let result = zero.run(data, decoder)\n assert result == Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \n\n If you wish to decode a value that is more deeply nested within the dynamic\n data, see [`subfield`](#subfield) and [`at`](#at).\n\n If you wish to return a default in the event that a field is not present,\n see [`optional_field`](#optional_field) and / [`optionally_at`](#optionally_at).\n\nSynonyms:\ndecode/zero.field\ndecode/zero field","url":"decode/zero.html#field"},{"doc":"decode/zero","title":"list","content":"pub fn list(of inner: Decoder(a)) -&gt; Decoder(List(a))\n A decoder that decodes lists where all elements are decoded with a given\n decoder.\n\n # Examples\n\n \n let result =\n   zero.run(dynamic.from([1, 2, 3]), zero.list(of: zero.int))\n assert result == Ok([1, 2, 3])\n \n\nSynonyms:\ndecode/zero.list\ndecode/zero list","url":"decode/zero.html#list"},{"doc":"decode/zero","title":"map","content":"pub fn map(\n  decoder: Decoder(a),\n  transformer: fn(a) -&gt; b,\n) -&gt; Decoder(b)\n Apply a transformation function to any value decoded by the decoder.\n\n # Examples\n\n \n let decoder = zero.int |&gt; zero.map(int.to_string)\n let result = zero.run(dynamic.from(1000), decoder)\n assert result == Ok(&quot;1000&quot;)\n \n\nSynonyms:\ndecode/zero.map\ndecode/zero map","url":"decode/zero.html#map"},{"doc":"decode/zero","title":"map_errors","content":"pub fn map_errors(\n  decoder: Decoder(a),\n  transformer: fn(List(DecodeError)) -&gt; List(DecodeError),\n) -&gt; Decoder(a)\n Apply a transformation function to any errors returned by the decoder.\n\nSynonyms:\ndecode/zero.map_errors\ndecode/zero map_errors","url":"decode/zero.html#map_errors"},{"doc":"decode/zero","title":"new_primitive_decoder","content":"pub fn new_primitive_decoder(\n  decoding_function: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n  zero: a,\n) -&gt; Decoder(a)\n Create a decoder for a new data type from a decoding function.\n\n This function is used for new primitive types. For example, you might\n define a decoder for Erlang&#39;s pid type.\n\n A default &quot;zero&quot; value is also required to make a decoder. When this\n decoder is used as part of a larger decoder this zero value used as\n a placeholder so that the rest of the decoder can continue to run and\n collect all decoding errors.\n\n If you were to make a decoder for the `String` type (rather than using the\n build-in `string` decoder) you would define it like so:\n\n \n import gleam/dynamic\n import decode/zero\n\n pub fn string_decoder() -&gt; zero.Decoder(String) {\n   zero.new_primitive_decoder(dynamic.string, &quot;&quot;)\n }\n \n\nSynonyms:\ndecode/zero.new_primitive_decoder\ndecode/zero new_primitive_decoder","url":"decode/zero.html#new_primitive_decoder"},{"doc":"decode/zero","title":"one_of","content":"pub fn one_of(\n  first: Decoder(a),\n  or alternatives: List(Decoder(a)),\n) -&gt; Decoder(a)\n Create a new decoder from several other decoders. Each of the inner\n decoders is run in turn, and the value from the first to succeed is used.\n\n If no decoder succeeds then the errors from the first decoder is used.\n If you wish for different errors then you may wish to use the\n `collapse_errors` or `map_errors` functions.\n\n # Examples\n\n \n zero.one_of(zero.string, or: [\n   zero.int |&gt; zero.map(int.to_string),\n   zero.float |&gt; zero.map(float.to_string),\n ])\n |&gt; zero.run(dynamic.from(1000))\n // -&gt; Ok(&quot;1000&quot;)\n \n\nSynonyms:\ndecode/zero.one_of\ndecode/zero one_of","url":"decode/zero.html#one_of"},{"doc":"decode/zero","title":"optional","content":"pub fn optional(inner: Decoder(a)) -&gt; Decoder(Option(a))\n A decoder that decodes nullable values of a type decoded by with a given\n decoder.\n\n This function can handle common representations of null on all runtimes, such as\n `nil`, `null`, and `undefined` on Erlang, and `undefined` and `null` on\n JavaScript.\n\n # Examples\n\n \n let result = zero.run(dynamic.from(100), zero.optional(zero.int))\n assert result == Ok(option.Some(100))\n \n\n \n let result = zero.run(dynamic.from(Nil), zero.optional(zero.int))\n assert result == Ok(option.None)\n \n\nSynonyms:\ndecode/zero.optional\ndecode/zero optional","url":"decode/zero.html#optional"},{"doc":"decode/zero","title":"optional_field","content":"pub fn optional_field(\n  key: a,\n  default: b,\n  field_decoder: Decoder(b),\n  next: fn(b) -&gt; Decoder(c),\n) -&gt; Decoder(c)\n Run a decoder on a field of a `Dynamic` value, decoding the value if it is\n of the desired type, or returning errors. The given default value is\n returned if there is no field for the specified key.\n\n This function will index into dictionaries with any key type, and if the key is\n an int then it&#39;ll also index into Erlang tuples and JavaScript arrays, and\n the first two elements of Gleam lists.\n\n # Examples\n\n \n let data = dynamic.from(dict.from_list([\n   #(&quot;name&quot;, &quot;Lucy&quot;),\n ]))\n\n let decoder = {\n   use name &lt;- zero.field(&quot;name&quot;, string)\n   use email &lt;- zero.optional_field(&quot;email&quot;, &quot;n/a&quot;, string)\n   SignUp(name: name, email: email)\n }\n\n let result = zero.run(data, decoder)\n assert result == Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;n/a&quot;))\n \n\nSynonyms:\ndecode/zero.optional_field\ndecode/zero optional_field","url":"decode/zero.html#optional_field"},{"doc":"decode/zero","title":"optionally_at","content":"pub fn optionally_at(\n  path: List(a),\n  default: b,\n  inner: Decoder(b),\n) -&gt; Decoder(b)\n A decoder that decodes a value that is nested within other values. For\n example, decoding a value that is within some deeply nested JSON objects.\n\n This function will index into dictionaries with any key type, and if the key is\n an int then it&#39;ll also index into Erlang tuples and JavaScript arrays, and\n the first two elements of Gleam lists.\n\n # Examples\n\n \n let decoder = zero.optionally_at([&quot;one&quot;, &quot;two&quot;], 100, zero.int)\n\n let data = dynamic.from(dict.from_list([\n   #(&quot;one&quot;, dict.from_list([])),\n ]))\n\n\n zero.run(data, decoder)\n // -&gt; Ok(100)\n \n\nSynonyms:\ndecode/zero.optionally_at\ndecode/zero optionally_at","url":"decode/zero.html#optionally_at"},{"doc":"decode/zero","title":"run","content":"pub fn run(\n  data: Dynamic,\n  decoder: Decoder(a),\n) -&gt; Result(a, List(DecodeError))\n Run a decoder on a `Dynamic` value, decoding the value if it is of the\n desired type, or returning errors.\n\n # Examples\n\n \n let decoder = {\n   use name &lt;- zero.field(&quot;email&quot;, zero.string)\n   use email &lt;- zero.field(&quot;password&quot;, zero.string)\n   zero.success(SignUp(name: name, email: email))\n }\n\n zero.run(data, decoder)\n \n\nSynonyms:\ndecode/zero.run\ndecode/zero run","url":"decode/zero.html#run"},{"doc":"decode/zero","title":"subfield","content":"pub fn subfield(\n  field_path: List(a),\n  field_decoder: Decoder(b),\n  next: fn(b) -&gt; Decoder(c),\n) -&gt; Decoder(c)\n The same as [`field`](#field), except taking a path to the value rather\n than a field name.\n\n This function will index into dictionaries with any key type, and if the key is\n an int then it&#39;ll also index into Erlang tuples and JavaScript arrays, and\n the first two elements of Gleam lists.\n\n # Examples\n\n \n let data = dynamic.from(dict.from_list([\n   #(&quot;data&quot;, dict.from_list([\n     #(&quot;email&quot;, &quot;lucy@example.com&quot;),\n     #(&quot;name&quot;, &quot;Lucy&quot;),\n   ]))\n ]))\n\n let decoder = {\n   use name &lt;- zero.subfield([&quot;data&quot;, &quot;name&quot;], zero.string)\n   use email &lt;- zero.subfield([&quot;data&quot;, &quot;email&quot;], zero.string)\n   zero.success(SignUp(name: name, email: email))\n }\n let result = zero.run(data, decoder)\n assert result == Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \n\nSynonyms:\ndecode/zero.subfield\ndecode/zero subfield","url":"decode/zero.html#subfield"},{"doc":"decode/zero","title":"success","content":"pub fn success(data: a) -&gt; Decoder(a)\n Finalise a decoder having successfully extracted a value.\n\n # Examples\n\n \n let data = dynamic.from(dict.from_list([\n   #(&quot;email&quot;, &quot;lucy@example.com&quot;),\n   #(&quot;name&quot;, &quot;Lucy&quot;),\n ]))\n\n let decoder = {\n   use name &lt;- zero.field(&quot;name&quot;, string)\n   use email &lt;- zero.field(&quot;email&quot;, string)\n   zero.success(SignUp(name: name, email: email))\n }\n\n let result = zero.run(data, decoder)\n assert result == Ok(SignUp(name: &quot;Lucy&quot;, email: &quot;lucy@example.com&quot;))\n \n\nSynonyms:\ndecode/zero.success\ndecode/zero success","url":"decode/zero.html#success"},{"doc":"decode/zero","title":"then","content":"pub fn then(\n  decoder: Decoder(a),\n  next: fn(a) -&gt; Decoder(b),\n) -&gt; Decoder(b)\n Create a new decoder based upon the value of a previous decoder.\n\n This may be useful to run one previous decoder to use in further decoding.\n\nSynonyms:\ndecode/zero.then\ndecode/zero then","url":"decode/zero.html#then"},{"doc":"decode/zero","title":"decode/zero","content":" The `Dynamic` type is used to represent dynamically typed data. That is, data\n that we don&#39;t know the precise type of yet, so we need to introspect the data to\n see if it is of the desired type before we can use it. Typically data like this\n would come from user input or from untyped languages such as Erlang or JavaScript.\n\n This module provides the `Decoder` type and associated functions, which provides\n a type-safe and composable way to convert dynamic data into some desired type,\n or into errors if the data doesn&#39;t have the desired structure.\n\n The `Decoder` type is generic and has 1 type parameter, which is the type that\n it attempts to zero. A `Decoder(String)` can be used to decode strings, and a\n `Decoder(Option(Int))` can be used to decode `Option(Int)`s\n\n Decoders work using _runtime reflection_ and the data structures of the target\n platform. Differences between Erlang and JavaScript data structures may impact\n your decoders, so it is important to test your decoders on all supported\n platforms.\n\n The decoding technique used by this module was inspired by Juraj Petráš&#39;\n [Toy](https://github.com/Hackder/toy), Go&#39;s `encoding/json`, and Elm&#39;s\n `Json.Decode`. Thank you to them!\n\n # Examples\n\n Dynamic data may come from various sources and so many different syntaxes could\n be used to describe or construct them. In these examples a pseudocode\n syntax is used to describe the data.\n\n ## Simple types\n\n This module defines decoders for simple data types such as [`string`](#string),\n [`int`](#int), [`float`](#float), [`bit_array`](#bit_array), and [`bool`](#bool).\n\n ```gleam\n // Data:\n // &quot;Hello, Joe!&quot;\n\n let result = zero.run(data, zero.string)\n assert result == Ok(&quot;Hello, Joe!&quot;)\n ```\n\n ## Lists\n\n The [`list`](#list) decoder decodes `List`s. To use it you must construct it by\n passing in another decoder into the `list` function, which is the decoder that\n is to be used for the elements of the list, type checking both the list and its\n elements.\n\n ```gleam\n // Data:\n // [1, 2, 3, 4]\n\n let result = zero.run(data, zero.list(zero.int))\n assert result == Ok([1, 2, 3, 4])\n ```\n\n On Erlang this decoder can decode from lists, and on JavaScript it can\n decode from lists as well as JavaScript arrays.\n\n ## Options\n\n The [`optional`](#optional) decoder is used to decode values that may or may not\n be present. In other environment these might be called &quot;nullable&quot; values.\n\n Like the `list` decoder, the `optional` decoder takes another decoder,\n which is used to decode the value if it is present.\n\n ```gleam\n // Data:\n // 12.45\n\n let result = zero.run(data, zero.optional(zero.int))\n assert result == Ok(option.Some(12.45))\n ```\n ```gleam\n // Data:\n // null\n\n let result = zero.run(data, zero.optional(zero.int))\n assert result == Ok(option.None)\n ```\n\n This decoder knows how to handle multiple different runtime representations of\n absent values, including `Nil`, `None`, `null`, and `undefined`.\n\n ## Dicts\n\n The [`dict`](#dict) decoder decodes `Dicts` and contains two other decoders, one\n for the keys, one for the values.\n\n ```gleam\n // Data:\n // { &quot;Lucy&quot; -&gt; 10, &quot;Nubi&quot; -&gt; 20 }\n\n let result = zero.run(data, zero.dict(zero.string, zero.int))\n assert result == Ok(dict.from_list([\n   #(&quot;Lucy&quot;, 10),\n   #(&quot;Nubi&quot;, 20),\n ]))\n ```\n\n ## Indexing objects\n\n The [`at`](#at) decoder can be used to decode a value that is nested within\n key-value containers such as Gleam dicts, Erlang maps, or JavaScript objects.\n\n ```gleam\n // Data:\n // { &quot;one&quot; -&gt; { &quot;two&quot; -&gt; 123 } }\n\n let result = zero.run(data, zero.at([&quot;one&quot;, &quot;two&quot;], zero.int))\n assert result == Ok(123)\n ```\n\n ## Indexing arrays\n\n If you use ints as keys then the [`at`](#at) decoder can be used to index into\n array-like containers such as Gleam or Erlang tuples, or JavaScript arrays.\n\n ```gleam\n // Data:\n // [&quot;one&quot;, &quot;two&quot;, &quot;three&quot;]\n\n let result = zero.run(data, zero.at([1], zero.string))\n assert result == Ok(&quot;two&quot;)\n ```\n\n ## Records\n\n Decoding records from dynamic data is more complex and requires combining a\n decoder for each field and a special constructor that builds your records with\n the decoded field values.\n\n ```gleam\n // Data:\n // {\n //   &quot;score&quot; -&gt; 180,\n //   &quot;name&quot; -&gt; &quot;Mel Smith&quot;,\n //   &quot;is-admin&quot; -&gt; false,\n //   &quot;enrolled&quot; -&gt; true,\n //   &quot;colour&quot; -&gt; &quot;Red&quot;,\n // }\n\n let decoder = {\n   use name &lt;- zero.field(&quot;name&quot;, zero.string)\n   use score &lt;- zero.field(&quot;score&quot;, zero.int)\n   use colour &lt;- zero.field(&quot;colour&quot;, zero.string)\n   use enrolled &lt;- zero.field(&quot;enrolled&quot;, zero.bool)\n   zero.success(Player(name:, score:, colour:, enrolled:))\n }\n\n let result = zero.run(data, decoder)\n assert result == Ok(Player(&quot;Mel Smith&quot;, 180, &quot;Red&quot;, True))\n ```\n\n ## Enum variants\n\n Imagine you have a custom type where all the variants do not contain any values.\n\n ```gleam\n pub type PocketMonsterType {\n   Fire\n   Water\n   Grass\n   Electric\n }\n ```\n\n You might choose to encode these variants as strings, `&quot;fire&quot;` for `Fire`,\n `&quot;water&quot;` for `Water`, and so on. To decode them you&#39;ll need to decode the dynamic\n data as a string, but then you&#39;ll need to decode it further still as not all\n strings are valid values for the enum. This can be done with the `then`\n function, which enables running a second decoder after the first one\n succeeds.\n\n ```gleam\n let decoder = {\n   use decoded_string &lt;- zero.then(zero.string)\n   case decoded_string {\n     // Return succeeding decoders for valid strings\n     &quot;fire&quot; -&gt; zero.success(Fire)\n     &quot;water&quot; -&gt; zero.success(Water)\n     &quot;grass&quot; -&gt; zero.success(Grass)\n     &quot;electric&quot; -&gt; zero.success(Electric)\n     // Return a failing decoder for any other strings\n     _ -&gt; zero.failure(Fire, &quot;PocketMonsterType&quot;)\n   }\n }\n\n let result = zero.run(dynamic.from(&quot;water&quot;), decoder)\n assert result == Ok(Water)\n\n let result = zero.run(dynamic.from(&quot;wobble&quot;), decoder)\n assert result == Error([DecodeError(&quot;PocketMonsterType&quot;, &quot;String&quot;, [])])\n ```\n\n ## Record variants\n\n Decoding type variants that contain other values is done by combining the\n techniques from the &quot;enum variants&quot; and &quot;records&quot; examples. Imagine you have\n this custom type that you want to decode:\n\n ```gleam\n pub type PocketMonsterPerson {\n   Trainer(name: String, badge_count: Int)\n   GymLeader(name: String, speciality: PocketMonsterType)\n }\n ```\n And you would like to be able to decode these from dynamic data like this:\n ```erlang\n {\n   &quot;type&quot; -&gt; &quot;trainer&quot;,\n   &quot;name&quot; -&gt; &quot;Ash&quot;,\n   &quot;badge-count&quot; -&gt; 1,\n }\n ```\n ```erlang\n {\n   &quot;type&quot; -&gt; &quot;gym-leader&quot;,\n   &quot;name&quot; -&gt; &quot;Misty&quot;,\n   &quot;speciality&quot; -&gt; &quot;water&quot;,\n }\n ```\n\n Notice how both documents have a `&quot;type&quot;` field, which is used to indicate which\n variant the data is for.\n\n First, define decoders for each of the variants:\n\n ```gleam\n let trainer_decoder = {\n   use name &lt;- zero.field(&quot;name&quot;, zero.string)\n   use badge_count &lt;- zero.field(&quot;badge-count&quot;, zero.int)\n   zero.success(Trainer(name, badge_count))\n })\n\n let gym_leader_decoder = {\n   use name &lt;- zero.field(&quot;name&quot;, zero.string)\n   use speciality &lt;- zero.field(&quot;speciality&quot;, pocket_monster_type_decoder)\n   zero.success(GymLeader(name, speciality))\n }\n ```\n\n A third decoder can be used to extract and decode the `&quot;type&quot;` field, and the\n `then` function then returns whichever decoder is suitable for the document.\n\n ```gleam\n let decoder = {\n   use tag &lt;- zero.field(&quot;type&quot;, zero.string)\n   case tag {\n     &quot;gym-leader&quot; -&gt; gym_leader_decoder\n     _ -&gt; trainer_decoder\n   }\n }\n\n zero.run(data, decoder)\n ```","url":"decode/zero.html"}]);